@@ -2,12 +2,14 @@
 from __future__ import annotations
 
 import json
+import os
 import sys
 import time
 import urllib.request
-import subprocess  # NEW: for optional git clone
-import tempfile  # NEW
-import shutil  # NEW
+from urllib.parse import urlparse
+import subprocess
+import tempfile
+import shutil
 from pathlib import Path
 from typing import Any, Dict, List, Optional, Tuple
 
@@ -152,7 +154,6 @@ def _is_prerelease(v: Any) -> bool:
 
         if isinstance(v, Version):
             return bool(v.is_prerelease)
-        # if str passed
         return Version(str(v)).is_prerelease
     except Exception:
         return False
@@ -162,7 +163,6 @@ def _pick_best_in_bucket(cands: List[Tuple[Any, Dict[str, Any]]]) -> Dict[str, A
     """Prefer stable > pre-release; within each, choose highest version."""
     if not cands:
         return {}
-    # stable first
     stable: List[Tuple[Any, Dict[str, Any]]] = []
     pre: List[Tuple[Any, Dict[str, Any]]] = []
     for vkey, it in cands:
@@ -170,7 +170,6 @@ def _pick_best_in_bucket(cands: List[Tuple[Any, Dict[str, Any]]]) -> Dict[str, A
     bucket = stable or pre
     if not bucket:
         return {}
-    # highest version (desc)
     bucket.sort(key=lambda x: x[0], reverse=True)
     return bucket[0][1]
 
@@ -211,10 +210,7 @@ def _choose_best_candidate(
 
 
 def _is_dns_or_conn_failure(err: Exception) -> bool:
-    """
-    Heuristic: detect common DNS/connection failures in message chain.
-    Avoids importing requests/urllib3; checks text only.
-    """
+    """Heuristic: detect common DNS/connection failures by message text."""
     needles = (
         "temporary failure in name resolution",
         "name or service not known",
@@ -237,11 +233,43 @@ def _is_dns_or_conn_failure(err: Exception) -> bool:
     return False
 
 
+def _env_on(name: str) -> bool:
+    v = (os.getenv(name) or "").strip().lower()
+    return v in {"1", "true", "yes", "on"}
+
+
+def _env_on_default_true(name: str) -> bool:
+    """
+    True if env var is unset or truthy; False only if explicitly disabled (0/false/no/off).
+    """
+    raw = os.getenv(name)
+    if raw is None or raw.strip() == "":
+        return True
+    return raw.strip().lower() in {"1", "true", "yes", "on"}
+
+
+def _json_pretty(obj: Any) -> str:
+    try:
+        return json.dumps(obj, indent=2, ensure_ascii=False, default=str)
+    except Exception:
+        try:
+            return json.dumps(_to_dict(obj), indent=2, ensure_ascii=False, default=str)
+        except Exception:
+            return repr(obj)
+
+
+def _json_preview(obj: Any, *, limit: int = 6000) -> str:
+    s = _json_pretty(obj)
+    return (
+        s if len(s) <= limit else f"{s[:limit]}\n… (truncated {len(s) - limit} chars)"
+    )
+
+
 # ------------------------- Tiny on-disk resolver cache -------------------------
 
 
 def _cache_path(cfg) -> Path:
-    # ~/.matrix/cache/resolve.json  (portable; creates dirs as needed)
+    # ~/.matrix/cache/resolve.json
     root = Path(cfg.home).expanduser()
     cdir = root / "cache"
     try:
@@ -287,9 +315,7 @@ def _cache_put(cfg, raw: str, fqid: str) -> None:
         data = {"hub": str(cfg.hub_base), "entries": {}}
     entries: Dict[str, Any] = data.setdefault("entries", {})
     entries[raw] = {"fqid": fqid, "ts": time.time()}
-    # keep last ~100 to bound size
     if len(entries) > 120:
-        # prune oldest ~40
         keys_sorted = sorted(entries.items(), key=lambda kv: kv[1].get("ts", 0))
         for k, _ in keys_sorted[:40]:
             entries.pop(k, None)
@@ -302,14 +328,6 @@ def _cache_put(cfg, raw: str, fqid: str) -> None:
 def _resolve_fqid_via_search(client, cfg, raw_id: str) -> str:  # pragma: no cover
     """
     Resolve a short/raw id to a fully-qualified id (ns:name@version) with minimal traffic.
-
-    Strategy:
-      • If already fqid -> return raw_id.
-      • Cache hit -> return.
-      • One search with (type=ns or 'mcp_server'), include_pending=True (so dev catalogs resolve offline).
-      • If no candidates and ns missing -> one broadened search without type (last resort).
-      • Choose best: prefer type 'mcp_server', then latest (stable > pre), else any type latest.
-      • On public-hub DNS/conn failure -> try once against http://localhost:443.
     """
     if _is_fqid(raw_id):
         return raw_id
@@ -326,22 +344,18 @@ def _resolve_fqid_via_search(client, cfg, raw_id: str) -> str:  # pragma: no cov
         params: Dict[str, Any] = {
             "q": want_name,
             "limit": 25,
-            "include_pending": True,  # so dev/local catalogs work offline
+            "include_pending": True,
         }
-        # default to mcp_server if ns not provided and not broadening yet
         if ns_hint and not broaden:
             params["type"] = ns_hint
         elif (ns_hint is None) and (not broaden):
             params["type"] = "mcp_server"
-        # broadened call removes type filter
         payload = cli.search(**params)
         return _items_from(payload)
 
-    # primary call (typed or mcp_server bias)
     try:
         items = _search_once(client, ns_hint=want_ns, broaden=False)
     except Exception as e:
-        # try localhost once if public hub unreachable
         if _is_dns_or_conn_failure(e):
             try:
                 from matrix_sdk.client import MatrixClient as _MC
@@ -353,7 +367,6 @@ def _resolve_fqid_via_search(client, cfg, raw_id: str) -> str:  # pragma: no cov
                 )
             except Exception:
                 raise
-
         else:
             raise
 
@@ -361,12 +374,10 @@ def _resolve_fqid_via_search(client, cfg, raw_id: str) -> str:  # pragma: no cov
         items, want_ns=want_ns, want_name=want_name, want_ver=want_ver
     )
 
-    # If no candidate and ns missing, broaden (one extra query only when needed)
     if not best and want_ns is None:
         try:
             items2 = _search_once(client, ns_hint=None, broaden=True)
         except Exception:
-            # ignore and leave best as None
             items2 = []
         best = _choose_best_candidate(
             items2, want_ns=want_ns, want_name=want_name, want_ver=want_ver
@@ -406,10 +417,7 @@ def _sanitize_segment(s: str, fallback: str = "unnamed") -> str:
 
 
 def _label_from_fqid_alias(fqid: str, alias: str) -> str:
-    """
-    Build the server-safe plan label <alias>/<version> from fqid and alias.
-    Never include client paths; sanitize both parts to be cross-platform safe.
-    """
+    """Build the server-safe plan label <alias>/<version> from fqid and alias."""
     ver = fqid.rsplit("@", 1)[-1] if "@" in fqid else "0"
     return f"{_sanitize_segment(alias)}/{_sanitize_segment(ver)}"
 
@@ -428,6 +436,74 @@ def _ensure_local_writable(path: Path) -> None:
             pass
 
 
+def _summarize_outcome(outcome: Dict[str, Any]) -> Dict[str, Any]:
+    plan = outcome.get("plan") if isinstance(outcome.get("plan"), dict) else {}
+    lockfile = (
+        outcome.get("lockfile") if isinstance(outcome.get("lockfile"), dict) else {}
+    )
+    prov_urls: List[str] = []
+    try:
+        ents = lockfile.get("entities") or []
+        for e in ents:
+            u = ((e or {}).get("provenance") or {}).get("source_url")
+            if isinstance(u, str) and u.strip():
+                prov_urls.append(u.strip())
+    except Exception:
+        pass
+    return {
+        "has_plan": bool(plan),
+        "plan_keys": sorted(list(plan.keys())) if isinstance(plan, dict) else [],
+        "plan_artifacts_len": (
+            len(plan.get("artifacts", []) or []) if isinstance(plan, dict) else 0
+        ),
+        "lockfile_provenance_urls": prov_urls,
+    }
+
+
+def _log_outcome_preview_and_dump(outcome: Dict[str, Any], tgt_path: Path) -> None:
+    if _env_on("MATRIX_SDK_VERBOSE_PLAN"):
+        info("Hub outcome (preview):\n" + _json_preview(outcome, limit=6000))
+    if _env_on("MATRIX_SDK_DUMP_PLAN"):
+        try:
+            dump = tgt_path / "_hub_outcome.json"
+            dump.write_text(_json_pretty(outcome), encoding="utf-8")
+            info(f"Wrote full Hub outcome → {dump}")
+        except Exception as e:
+            warn(f"Could not write _hub_outcome.json ({e})")
+
+
+def _post_build_runner_hint(tgt_path: Path) -> None:
+    """
+    After building, print hints if the runner is a connector (PID=0 / no port).
+    This helps users understand why `matrix run` won't expose a port.
+    """
+    try:
+        rj = tgt_path / "runner.json"
+        if not rj.is_file():
+            return
+        runner = json.loads(rj.read_text(encoding="utf-8"))
+        rtype = (runner.get("type") or "").strip().lower()
+        if rtype != "connector":
+            return
+        url = (runner.get("url") or "").strip()
+        if url:
+            info(f"Connector runner detected (no local process). URL → {url}")
+            host = ""
+            try:
+                host = urlparse(url).hostname or ""
+            except Exception:
+                pass
+            if host in {"127.0.0.1", "localhost"}:
+                warn(
+                    "This connector points to localhost. Ensure the MCP server is running at that URL. "
+                    "Tip: use `matrix mcp probe --url <url>`.\n"
+                    "Note: `matrix run` for connectors shows PID=0 and does not expose a port."
+                )
+    except Exception:
+        # non-fatal
+        pass
+
+
 def _build_via_safe_plan(
     client,
     installer,
@@ -436,38 +512,36 @@ def _build_via_safe_plan(
     target: str,
     alias: str,
     timeout: int = 900,
-    runner_url: str | None = None,  # NEW
-    repo_url: str | None = None,  # NEW
+    runner_url: str | None = None,
+    repo_url: str | None = None,
 ):
     """
     Perform install using a server *label* (<alias>/<version>) instead of a client absolute path.
-    Works even if the SDK installer isn't patched, because we call client.install(...) ourselves.
     """
-    # 1) Ensure local target is writable before network calls
     tgt_path = Path(target).expanduser().resolve()
     _ensure_local_writable(tgt_path)
 
-    # 2) Request plan from Hub with a safe label
     label = _label_from_fqid_alias(fqid, alias)
-    outcome = client.install(fqid, target=label)  # <-- no absolute path leakage
+    info(f"Requesting plan for {fqid} with label '{label}'")
+    outcome = _to_dict(client.install(fqid, target=label))
+    _log_outcome_preview_and_dump(outcome, tgt_path)
 
-    # 3) Materialize locally (files/artifacts/runner.json)
-    report = installer.materialize(_to_dict(outcome), tgt_path)
+    info("Materializing plan…")
+    report = installer.materialize(outcome, tgt_path)
 
-    # 3.1) (NEW) Post-materialize: fetch runner.json and/or clone repo if needed
+    # Post-materialize assist (runner/repo)
     try:
         _maybe_fetch_runner_and_repo(
             tgt_path,
-            report=_to_dict(outcome),
+            report=outcome,
             runner_url=runner_url,
             repo_url=repo_url,
         )
     except Exception as e:
         warn(f"post-materialize runner/repo step failed (non-fatal): {e}")
 
-    # 4) Load runner and prepare env (venv/node)
+    # Load runner + prepare env
     try:
-        # using the SDK helper if available; fall back to reading runner.json directly
         load = getattr(installer, "_load_runner_from_report", None)
         runner = (
             load(report, tgt_path) if callable(load) else _load_runner_direct(tgt_path)
@@ -476,6 +550,16 @@ def _build_via_safe_plan(
         runner = _load_runner_direct(tgt_path)
 
     installer.prepare_env(tgt_path, runner, timeout=timeout)
+
+    # Helpful summary & connector hint
+    s = _summarize_outcome(outcome)
+    info(
+        f"Install summary → files={getattr(report, 'files_written', 0)} "
+        f"artifacts={getattr(report, 'artifacts_fetched', 0)} "
+        f"plan.artifacts={s.get('plan_artifacts_len')} "
+        f"lockfile.provenance={s.get('lockfile_provenance_urls') or []}"
+    )
+    _post_build_runner_hint(tgt_path)
     return tgt_path
 
 
@@ -489,7 +573,7 @@ def _load_runner_direct(target_path: Path) -> Dict[str, Any]:
     return {}
 
 
-# ------------------------- Inline manifest helpers (new) -------------------------
+# ------------------------- Inline manifest helpers -------------------------
 
 
 def _looks_like_url(s: str) -> bool:  # pragma: no cover
@@ -504,38 +588,31 @@ def _load_manifest_from(source: str) -> tuple[Dict[str, Any], Optional[str]]:
     src = (source or "").strip()
     if not src:
         raise ValueError("empty manifest source")
-    # Simple loader without new deps: http(s) via urllib, file path via Path
     if src.lower().startswith("http://") or src.lower().startswith("https://"):
-        # Use stdlib only
-        with urllib.request.urlopen(src, timeout=10) as resp:  # nosec - user-provided dev URL
+        with urllib.request.urlopen(src, timeout=15) as resp:  # nosec - dev provided URL
             data = resp.read().decode("utf-8")
         return json.loads(data), src
     if src.lower().startswith("file://"):
         p = Path(src[7:])
         return json.loads(p.read_text(encoding="utf-8")), str(p.as_uri())
-    # treat as filesystem path
     p = Path(src).expanduser().resolve()
     return json.loads(p.read_text(encoding="utf-8")), None
 
 
 def _normalize_manifest_for_sse(manifest: Dict[str, Any]) -> Dict[str, Any]:
-    """Force .mcp_registration.server.url → /sse and remove 'transport' (non-destructive for other fields)."""
+    """Normalize SSE url and strip 'transport' if present (non-destructive)."""
     try:
         mcp = manifest.setdefault("mcp_registration", {})
         server = mcp.setdefault("server", {})
         url = (server.get("url") or "").strip()
         if url:
-            # strip trailing slashes then ensure exactly one '/sse'
             while url.endswith("/"):
                 url = url[:-1]
             if not url.endswith("/sse"):
                 url = f"{url}/sse"
             server["url"] = url
-        # drop transport if present (prevents /messages/ rewrites downstream)
-        if "transport" in server:
-            server.pop("transport", None)
+        server.pop("transport", None)
     except Exception:
-        # do not fail install on normalization; leave manifest unchanged
         pass
     return manifest
 
@@ -550,17 +627,14 @@ def _build_via_inline_manifest(
     target: str,
     alias: str,
     timeout: int = 900,
-    runner_url: str | None = None,  # NEW
-    repo_url: str | None = None,  # NEW
+    runner_url: str | None = None,
+    repo_url: str | None = None,
 ):
-    """Install using an inline manifest via client.install_manifest (non-destructive fallback if unavailable)."""
+    """Install using an inline manifest via client.install_manifest when available."""
     tgt_path = Path(target).expanduser().resolve()
     _ensure_local_writable(tgt_path)
 
-    # send label instead of absolute path
     label = _label_from_fqid_alias(fqid, alias)
-
-    # Duck-typed feature: prefer client.install_manifest if present
     install_manifest_fn = getattr(client, "install_manifest", None)
     if not callable(install_manifest_fn):
         raise RuntimeError(
@@ -569,17 +643,21 @@ def _build_via_inline_manifest(
         )
 
     body_provenance = {"source_url": provenance_url} if provenance_url else None
-    outcome = install_manifest_fn(
-        fqid, manifest=manifest, target=label, provenance=body_provenance
+    info(f"Sending inline manifest for {fqid} with label '{label}'")
+    outcome = _to_dict(
+        install_manifest_fn(
+            fqid, manifest=manifest, target=label, provenance=body_provenance
+        )
     )
+    _log_outcome_preview_and_dump(outcome, tgt_path)
 
-    report = installer.materialize(_to_dict(outcome), tgt_path)
+    info("Materializing inline manifest…")
+    report = installer.materialize(outcome, tgt_path)
 
-    # (NEW) Post-materialize: fetch runner.json and/or clone repo if needed
     try:
         _maybe_fetch_runner_and_repo(
             tgt_path,
-            report=_to_dict(outcome),
+            report=outcome,
             runner_url=runner_url,
             repo_url=repo_url,
         )
@@ -595,10 +673,19 @@ def _build_via_inline_manifest(
         runner = _load_runner_direct(tgt_path)
 
     installer.prepare_env(tgt_path, runner, timeout=timeout)
+
+    s = _summarize_outcome(outcome)
+    info(
+        f"Install summary → files={getattr(report, 'files_written', 0)} "
+        f"artifacts={getattr(report, 'artifacts_fetched', 0)} "
+        f"plan.artifacts={s.get('plan_artifacts_len')} "
+        f"lockfile.provenance={s.get('lockfile_provenance_urls') or []}"
+    )
+    _post_build_runner_hint(tgt_path)
     return tgt_path
 
 
-# ------------------------- Runner & repo helpers (NEW) -------------------------
+# ------------------------- Runner & repo helpers -------------------------
 
 
 def _valid_runner_schema(obj: Dict[str, Any]) -> bool:
@@ -627,12 +714,24 @@ def _maybe_fetch_runner_and_repo(
 ) -> None:
     """
     Make installs 'just work' when Hub doesn't provide artifacts:
+
       • If --runner-url is provided: ALWAYS fetch into runner.json (backup if exists).
-      • Else if no runner.json and plan.runner_url exists: fetch it.
-      • If --repo-url is provided and the runner points to a missing entry file:
-         clone the repo into the target (excluding .git).
+      • Else: try plan.runner_url, then ALWAYS fetch lockfile.provenance.source_url → manifest.runner.
+      • If a connector runner already exists, replace it with a non-connector (python/node) runner
+        from the manifest when MATRIX_CLI_PREFER_MANIFEST_RUNNER (default ON).
+      • Clone repository:
+          - If --repo-url provided → use that.
+          - Else, if manifest.repository (or repositories[0]) exists and the runner’s entry file is missing → clone.
+            Respect 'subdir'/'subdirectory'/'path' if present in the manifest repo spec.
+      • Controlled by env:
+          - MATRIX_CLI_PREFER_MANIFEST_RUNNER (default: 1 / ON)
+          - MATRIX_CLI_AUTO_CLONE (default: 1 / ON)
+
     Non-fatal on failures; logs warnings.
     """
+    prefer_manifest_runner = _env_on_default_true("MATRIX_CLI_PREFER_MANIFEST_RUNNER")
+    auto_clone = _env_on_default_true("MATRIX_CLI_AUTO_CLONE")
+
     tgt_path.mkdir(parents=True, exist_ok=True)
     rpath = tgt_path / "runner.json"
 
@@ -647,12 +746,42 @@ def _maybe_fetch_runner_and_repo(
         rpath.write_text(json.dumps(obj, indent=2), encoding="utf-8")
         info(f"runner.json written → {rpath}")
 
-    # 1) Fetch runner from CLI flag (strong override)
+    def _current_runner() -> Dict[str, Any]:
+        try:
+            return json.loads(rpath.read_text(encoding="utf-8"))
+        except Exception:
+            return {}
+
+    def _first_provenance_url(rep: Dict[str, Any] | None) -> str:
+        if not isinstance(rep, dict):
+            return ""
+        try:
+            lockfile = rep.get("lockfile") or {}
+            for e in lockfile.get("entities") or []:
+                u = ((e or {}).get("provenance") or {}).get("source_url")
+                if isinstance(u, str) and u.strip():
+                    return u.strip()
+        except Exception:
+            pass
+        return ""
+
+    def _fetch_json(url: str, timeout: int = 20) -> Dict[str, Any]:
+        with urllib.request.urlopen(url, timeout=timeout) as resp:
+            return json.loads(resp.read().decode("utf-8"))
+
+    def _manifest_repo_spec(mf: Dict[str, Any]) -> Dict[str, Any] | None:
+        repo = mf.get("repository")
+        if isinstance(repo, dict):
+            return repo
+        repos = mf.get("repositories")
+        if isinstance(repos, list) and repos and isinstance(repos[0], dict):
+            return repos[0]
+        return None
+
+    # 1) Strong override: --runner-url
     if (runner_url or "").strip():
         try:
-            with urllib.request.urlopen(runner_url, timeout=15) as resp:
-                data = resp.read().decode("utf-8")
-            obj = json.loads(data)
+            obj = _fetch_json(runner_url)
             if _valid_runner_schema(obj):
                 _write_runner(obj)
             else:
@@ -662,14 +791,12 @@ def _maybe_fetch_runner_and_repo(
         except Exception as e:
             warn(f"--runner-url: failed to fetch runner.json ({e})")
 
-    # 2) Else if no runner.json and plan provided a runner_url
+    # 2) Plan hint (runner_url) if we still don't have a runner
     elif not rpath.exists():
         url = _plan_runner_url(report or {})
         if url:
             try:
-                with urllib.request.urlopen(url, timeout=15) as resp:
-                    data = resp.read().decode("utf-8")
-                obj = json.loads(data)
+                obj = _fetch_json(url)
                 if _valid_runner_schema(obj):
                     _write_runner(obj)
                 else:
@@ -679,48 +806,103 @@ def _maybe_fetch_runner_and_repo(
             except Exception as e:
                 warn(f"plan.runner_url: failed to fetch runner.json ({e})")
 
-    # 3) Optionally clone repo if needed and asked for
-    if (repo_url or "").strip():
-        need_clone = False
+    # 3) Manifest via lockfile provenance: ALWAYS try (even if a connector runner.json exists)
+    manifest: Dict[str, Any] | None = None
+    prov_url = _first_provenance_url(report or {})
+    if prov_url:
         try:
-            obj = json.loads(rpath.read_text(encoding="utf-8"))
+            manifest = _fetch_json(prov_url)
+        except Exception as e:
+            warn(f"provenance.manifest: failed to fetch manifest ({e})")
+
+    # If a manifest runner exists and is valid, prefer it for non-connector execution.
+    if manifest:
+        mf_runner = manifest.get("runner") or {}
+        if mf_runner and _valid_runner_schema(mf_runner):
+            cur = _current_runner()
+            cur_type = (cur.get("type") or "").strip().lower()
+            mf_type = (mf_runner.get("type") or "").strip().lower()
+            if (not rpath.exists()) or (
+                prefer_manifest_runner
+                and cur_type == "connector"
+                and mf_type in {"python", "node"}
+            ):
+                info("Using runner from manifest (writing/replacing runner.json).")
+                _write_runner(mf_runner)
+
+    # 4) Clone repository if the runner requires an entry file that isn't present.
+    #    Decide URL: explicit --repo-url > manifest.repository.url/repo
+    clone_from = (repo_url or "").strip()
+    mf_repo_spec = None
+    if not clone_from and manifest:
+        mf_repo_spec = _manifest_repo_spec(manifest)
+        if isinstance(mf_repo_spec, dict):
+            clone_from = (
+                mf_repo_spec.get("url") or mf_repo_spec.get("repo") or ""
+            ).strip()
+
+    # Determine if we need code based on the (possibly replaced) runner.json
+    try:
+        runner_obj = _current_runner()
+    except Exception:
+        runner_obj = {}
+
+    rtype = (runner_obj.get("type") or "").strip().lower()
+    entry = (runner_obj.get("entry") or "").strip()
+    need_clone = bool(
+        rtype in {"python", "node"} and (not entry or not (tgt_path / entry).exists())
+    )
+
+    if clone_from and (need_clone and (auto_clone or bool(repo_url))):
+        try:
+            with tempfile.TemporaryDirectory() as tmpd:
+                # honor branch/tag ref if present in manifest repo spec
+                clone_cmd = ["git", "clone", "--depth=1"]
+                ref = ""
+                if isinstance(mf_repo_spec, dict):
+                    ref = (
+                        mf_repo_spec.get("ref")
+                        or mf_repo_spec.get("branch")
+                        or mf_repo_spec.get("tag")
+                        or ""
+                    ).strip()
+                if ref and ref.upper() != "HEAD":
+                    clone_cmd += ["--branch", ref]
+                clone_cmd += [clone_from, tmpd]
+                subprocess.run(clone_cmd, check=True)
+
+                # Respect subdir if present
+                src_dir = Path(tmpd)
+                if isinstance(mf_repo_spec, dict):
+                    sub = (
+                        mf_repo_spec.get("subdir")
+                        or mf_repo_spec.get("subdirectory")
+                        or mf_repo_spec.get("path")
+                        or ""
+                    ).strip()
+                    if sub:
+                        src_dir = src_dir / sub
+
+                for p in src_dir.iterdir():
+                    if p.name == ".git":
+                        continue
+                    dest = tgt_path / p.name
+                    if p.is_dir():
+                        shutil.copytree(p, dest, dirs_exist_ok=True)
+                    else:
+                        shutil.copy2(p, dest)
+            info(f"Repository cloned into {tgt_path}")
+        except Exception as e:
+            warn(f"repo clone: failed to clone into target ({e})")
+
+    # 5) If repo provided a runner.json, prefer it when current runner is invalid
+    if not _valid_runner_schema(_current_runner()):
+        try:
+            obj = _current_runner()
+            if _valid_runner_schema(obj):
+                info("Loaded runner from repository.")
         except Exception:
-            obj = {}
-
-        t = (obj.get("type") or "").strip().lower()
-        if t == "python":
-            entry = (obj.get("entry") or "").strip()
-            if not entry:
-                need_clone = True
-            else:
-                if not (tgt_path / entry).exists():
-                    need_clone = True
-        elif t == "node":
-            entry = (obj.get("entry") or "").strip()
-            if not entry or not (tgt_path / entry).exists():
-                need_clone = True
-        else:
-            # no runner or connector: if user asked to clone, allow it
-            need_clone = True
-
-        if need_clone:
-            try:
-                with tempfile.TemporaryDirectory() as tmpd:
-                    subprocess.run(
-                        ["git", "clone", "--depth=1", repo_url, tmpd], check=True
-                    )
-                    # copy into target (excluding .git)
-                    for p in Path(tmpd).iterdir():
-                        if p.name == ".git":
-                            continue
-                        dest = tgt_path / p.name
-                        if p.is_dir():
-                            shutil.copytree(p, dest, dirs_exist_ok=True)
-                        else:
-                            shutil.copy2(p, dest)
-                info(f"Repository cloned into {tgt_path}")
-            except Exception as e:
-                warn(f"--repo-url: failed to clone into target ({e})")
+            pass
 
 
 # ----------------------------------- CLI -----------------------------------
@@ -752,7 +934,6 @@ def main(
             "Accepted: http(s)://, file://, or filesystem path."
         ),
     ),
-    # NEW — works now, no Hub change required
     runner_url: str | None = typer.Option(
         None,
         "--runner-url",
@@ -775,20 +956,17 @@ def main(
     ),
 ) -> None:
     """
-    Install a component locally using the SDK — with safe planning to avoid server 500s
-    caused by leaking client absolute paths to the Hub.
+    Install a component locally using the SDK — with safe planning to avoid server 500s.
 
     New (non-breaking):
       • --manifest/--from lets you provide a manifest inline when Hub lacks source_url.
-      • Resolver prefers the namespace the user typed (tool:/mcp_server:/server:),
-        falling back to mcp_server.
-      • --runner-url and --repo-url let you fetch a runner.json and optionally clone code
-        even when the Hub plan doesn't include them (works today, no Hub change required).
+      • Resolver prefers the namespace the user typed (tool:/mcp_server:/server:), falling back to mcp_server.
+      • --runner-url and --repo-url let you fetch a runner.json and optionally clone code even when the plan
+        doesn't include them.
 
-    Exit codes:
-      0  success
-      3  alias collision (with --no-prompt or declined overwrite)
-      10 hub/network/build/resolve error
+    Env toggles (default ON):
+      • MATRIX_CLI_PREFER_MANIFEST_RUNNER=1  — prefer manifest runner over connector
+      • MATRIX_CLI_AUTO_CLONE=1              — auto-clone repo when entry file is missing
     """
     from matrix_sdk.alias import AliasStore
     from matrix_sdk.client import MatrixClient
@@ -797,10 +975,8 @@ def main(
 
     cfg = load_config()
     if hub:
-        # create a new Config instance with hub override
         cfg = type(cfg)(hub_base=hub, token=cfg.token, home=cfg.home)
 
-    # Client & installer
     client = client_from_config(cfg)
     installer = LocalInstaller(client)
 
@@ -808,16 +984,17 @@ def main(
     try:
         ns_input = id.split(":", 1)[0] if ":" in id else None
         prefer_ns = ns_input or "mcp_server"
+        info(f"Resolving '{id}' (prefer_ns={prefer_ns})…")
         try:
             res = resolve_fqid(
                 client, cfg, id, prefer_ns=prefer_ns, allow_prerelease=False
             )
         except TypeError:
-            # fallback for older CLIs where resolver lacks kwargs
             res = resolve_fqid(client, cfg, id)
         fqid = res.fqid
         if res.note:
             warn(res.note)
+        info(f"Resolved → {fqid}")
     except Exception as e:
         error(f"Could not resolve id '{id}': {e}")
         raise typer.Exit(10)
@@ -826,7 +1003,6 @@ def main(
     alias = alias or suggest_alias(fqid)
     target = target or target_for(fqid, alias=alias, cfg=cfg)
 
-    # alias collision handling (unchanged)
     store = AliasStore()
     existing = store.get(alias)
     if existing and not force:
@@ -843,15 +1019,14 @@ def main(
     # Primary path: inline manifest when provided; else default safe-plan path
     try:
         if manifest:
-            # Load + normalize + install inline
             try:
                 mf, src_url = _load_manifest_from(manifest)
                 mf = _normalize_manifest_for_sse(mf)
+                info(f"Loaded manifest from '{manifest}' (provenance={bool(src_url)})")
             except Exception as e:
                 error(f"Failed to load manifest from '{manifest}': {e}")
                 raise typer.Exit(10)
 
-            # Try primary hub, then fallback to localhost:443 on DNS/conn error
             try:
                 _build_via_inline_manifest(
                     client,
@@ -861,8 +1036,8 @@ def main(
                     provenance_url=src_url,
                     target=target,
                     alias=alias,
-                    runner_url=runner_url,  # NEW
-                    repo_url=repo_url,  # NEW
+                    runner_url=runner_url,
+                    repo_url=repo_url,
                 )
             except Exception as e:
                 if _is_dns_or_conn_failure(e):
@@ -882,15 +1057,14 @@ def main(
                             provenance_url=src_url,
                             target=target,
                             alias=alias,
-                            runner_url=runner_url,  # NEW
-                            repo_url=repo_url,  # NEW
+                            runner_url=runner_url,
+                            repo_url=repo_url,
                         )
                     except Exception:
                         raise
                 else:
                     raise
         else:
-            # legacy/default path — requires Hub to have a source_url recorded
             try:
                 _build_via_safe_plan(
                     client,
@@ -898,8 +1072,8 @@ def main(
                     fqid,
                     target=target,
                     alias=alias,
-                    runner_url=runner_url,  # NEW
-                    repo_url=repo_url,  # NEW
+                    runner_url=runner_url,
+                    repo_url=repo_url,
                 )
             except Exception as e:
                 if _is_dns_or_conn_failure(e):
@@ -917,8 +1091,8 @@ def main(
                             fqid,
                             target=target,
                             alias=alias,
-                            runner_url=runner_url,  # NEW
-                            repo_url=repo_url,  # NEW
+                            runner_url=runner_url,
+                            repo_url=repo_url,
                         )
                     except Exception:
                         raise
@@ -937,7 +1111,11 @@ def main(
         error(f"Install failed: {e}")
         raise typer.Exit(10)
 
-    store.set(alias, id=fqid, target=target)
+    # Save alias mapping to the new target
+    try:
+        store.set(alias, id=fqid, target=target)
+    except Exception as e:
+        warn(f"Install succeeded but failed to persist alias mapping: {e}")
     success(f"installed {fqid}")
     info(f"→ {target}")
     info(f"Next: matrix run {alias}")
